from __future__ import division

import torch
from torch_geometric.nn.pool.topk_pool import topk, filter_adj, TopKPooling


def test_topk():
    x = torch.Tensor([2, 4, 5, 6, 2, 9])
    batch = torch.tensor([0, 0, 1, 1, 1, 1])

    perm = topk(x, 0.5, batch)

    assert perm.tolist() == [1, 5, 3]
    assert x[perm].tolist() == [4, 9, 6]
    assert batch[perm].tolist() == [0, 1, 1]


def test_filter_adj():
    edge_index = torch.tensor([[0, 0, 1, 1, 2, 2, 3, 3],
                               [1, 3, 0, 2, 1, 3, 0, 2]])
    edge_attr = torch.Tensor([1, 2, 3, 4, 5, 6, 7, 8])
    perm = torch.tensor([2, 3])

    edge_index, edge_attr = filter_adj(edge_index, edge_attr, perm)
    assert edge_index.tolist() == [[0, 1], [1, 0]]
    assert edge_attr.tolist() == [6, 8]


def test_topk_pooling():
    in_channels = 16
    edge_index = torch.tensor([[0, 0, 0, 1, 1, 1, 2, 2, 2, 3, 3, 3],
                               [1, 2, 3, 0, 2, 3, 0, 1, 3, 0, 1, 2]])
    num_nodes = edge_index.max().item() + 1
    x = torch.randn((num_nodes, in_channels))

    pool = TopKPooling(in_channels, ratio=0.5)
    assert pool.__repr__() == 'TopKPooling(16, ratio=0.5)'

    x, edge_index, _, _, _ = pool(x, edge_index)
    assert x.size() == (num_nodes // 2, in_channels)
    assert edge_index.size() == (2, 2)
